@@ -170,6 +170,9 @@ void VGA_CaptureStartNextFrame(void);
 void VGA_CaptureMarkError(void);
 bool VGA_CaptureValidateCurrentFrame(void);
 
+extern bool enable_supermegazeux_256colortext;
+extern bool pc98_timestamp5c;
+
 bool                                VGA_PITsync = false;
 
 unsigned int                        vbe_window_granularity = 0;
@@ -304,6 +307,31 @@ void pc98_egc4a0_write(Bitu port,Bitu val,Bitu iolen);
 Bitu pc98_egc4a0_read_warning(Bitu port,Bitu iolen);
 void pc98_egc4a0_write_warning(Bitu port,Bitu val,Bitu iolen);
 
+/* ARTIC (A Relative Time Indication Counter) I/O read.
+ * Ports 0x5C and 0x5E.
+ * This is required for some MS-DOS drivers such as the OAK CD-ROM driver
+ * in order for them to time out properly instead of infinitely hang.
+ *
+ * "A 24-bit binary counter that counts up at 307.2KHz" */
+Bitu pc98_read_artic(Bitu port,Bitu iolen) {
+	Bitu count = ((Bitu)(PIC_FullIndex()/*milliseconds*/ * 307.2)) & (Bitu)0xFFFFFFul/*mask at 24 bits*/;
+	Bitu r = ~0ul;
+
+	if ((port&0xFFFEul) == 0x5C) /* bits 15:0 */
+		r = count & 0xFFFFul;
+	else if ((port&0xFFFEul) == 0x5E) /* bits 23:8 */
+		r = (count >> 8u) & 0xFFFFul;
+
+	if (iolen == 1) {
+		if (port&1) r >>= 8;
+		r &= 0xFF;
+	}
+
+//	LOG_MSG("ARTIC port %x read %x iolen %u",(unsigned int)port,(unsigned int)r,(unsigned int)iolen);
+
+	return r;
+}
+
 void page_flip_debug_notify() {
     if (enable_page_flip_debugging_marker)
         vga_page_flip_occurred = true;
@@ -314,6 +342,7 @@ void vsync_poll_debug_notify() {
         vga_3da_polled = true;
 }
 
+uint32_t HercBlend_2_Table[16];
 uint32_t CGA_2_Table[16];
 uint32_t CGA_4_Table[256];
 uint32_t CGA_4_HiRes_Table[256];
@@ -367,7 +396,7 @@ void VGA_DetermineMode(void) {
 //      display them just fine. The other is that checking for 2-color CGA mode entirely by
 //      whether video RAM is mapped to B8000h is a really lame way to go about it.
 //
-//      The only catch here is that a contributer (Wengier, I think?) tied a DOS/V CGA rendering
+//      The only catch here is that a contributor (Wengier, I think?) tied a DOS/V CGA rendering
 //      mode into M_CGA2 that we need to watch for.
 //
             else if ((vga.gfx.miscellaneous & 0x0c)==0x0c && J3_IsCga4Dcga()) VGA_SetMode(M_DCGA);
@@ -573,7 +602,7 @@ static void resetSize(Bitu /*val*/) {
 
 class VFRCRATE : public Program {
 public:
-    void Run(void) {
+    void Run(void) override {
         if (cmd->FindExist("/?", false)) {
 			WriteOut("Locks or unlocks the video refresh rate.\n\n");
 			WriteOut("VFRCRATE [SET [OFF|PAL|NTSC|rate]\n");
@@ -615,7 +644,7 @@ class CGASNOW : public Program {
 public:
     /*! \brief      Program entry point, when the command is run
      */
-    void Run(void) {
+    void Run(void) override {
         if (cmd->FindExist("/?", false)) {
 			WriteOut("Turns CGA snow emulation on or off.\n\n");
 			WriteOut("CGASNOW [ON|OFF]\n");
@@ -659,6 +688,7 @@ static inline unsigned int int_log2(unsigned int val) {
 
 extern bool pcibus_enable;
 extern int hack_lfb_yadjust;
+extern int hack_lfb_xadjust;
 extern uint8_t GDC_display_plane_wait_for_vsync;
 
 void VGA_VsyncUpdateMode(VGA_Vsync vsyncmode);
@@ -693,6 +723,14 @@ void VGA_Reset(Section*) {
 
     LOG(LOG_MISC,LOG_DEBUG)("VGA_Reset() reinitializing VGA emulation");
 
+    str = section->Get_string("enable supermegazeux tweakmode");
+    if (str == "1" || str == "true")
+        enable_supermegazeux_256colortext = true;
+    else if (str == "0" || str == "false")
+        enable_supermegazeux_256colortext = false;
+    else
+        enable_supermegazeux_256colortext = false; // TODO: Default true if emulating a chipset that supports SuperMegazeux 256-color text mode
+
     GDC_display_plane_wait_for_vsync = pc98_section->Get_bool("pc-98 buffer page flip");
 
     enable_pci_vga = section->Get_bool("pci vga");
@@ -730,6 +768,10 @@ void VGA_Reset(Section*) {
         enable_pci_vga = false;
     }
 
+    /* must not overlap system RAM */
+    if (S3_LFB_BASE < (MEM_TotalPages()*4096))
+        S3_LFB_BASE = (MEM_TotalPages()*4096);
+
     if (enable_pci_vga && has_pcibus_enable()) {
         /* must be 32MB aligned (PCI) */
         S3_LFB_BASE +=  0x0FFFFFFUL;
@@ -741,10 +783,6 @@ void VGA_Reset(Section*) {
         S3_LFB_BASE &= ~0xFFFFUL;
     }
 
-    /* must not overlap system RAM */
-    if (S3_LFB_BASE < (MEM_TotalPages()*4096))
-        S3_LFB_BASE = (MEM_TotalPages()*4096);
-
     /* if the constraints we imposed make it impossible to maintain the alignment required for PCI,
      * then just switch off PCI VGA emulation. */
     if (IS_VGA_ARCH && enable_pci_vga && has_pcibus_enable()) {
@@ -898,6 +936,7 @@ void VGA_Reset(Section*) {
     vga_ignore_extended_memory_bit = section->Get_bool("ignore extended memory bit");
     enable_vretrace_poll_debugging_marker = section->Get_bool("vertical retrace poll debug line");
     vga_double_buffered_line_compare = section->Get_bool("double-buffered line compare");
+    hack_lfb_xadjust = section->Get_int("vesa lfb pel scanline adjust");
     hack_lfb_yadjust = section->Get_int("vesa lfb base scanline adjust");
     allow_vesa_lowres_modes = section->Get_bool("allow low resolution vesa modes");
     allow_vesa_4bpp_packed = section->Get_bool("allow 4bpp packed vesa modes");
@@ -1180,6 +1219,22 @@ void VGA_Reset(Section*) {
         /* Generate tables */
         VGA_SetCGA2Table(0,1);
         VGA_SetCGA4Table(0,1,2,3);
+
+        if (machine == MCH_HERC || machine == MCH_MDA) {
+            /* alternate table for blend mode, expect mapping 0=black 1-7=gray 8=black 9-f=white
+             * for use with blend code that reads this table and sums the output with a delayed version of the same */
+            const uint8_t total[2] = {0x0,0x05}; /* blending will yield {0,5,10} which is sufficient to recreate the effect */
+            for (Bitu i=0;i<16u;i++) {
+                HercBlend_2_Table[i]=
+#ifdef WORDS_BIGENDIAN
+                    ((Bitu)total[(i >> 0u) & 1u] << 0u  ) | ((Bitu)total[(i >> 1u) & 1u] << 8u  ) |
+                    ((Bitu)total[(i >> 2u) & 1u] << 16u ) | ((Bitu)total[(i >> 3u) & 1u] << 24u );
+#else 
+                    ((Bitu)total[(i >> 3u) & 1u] << 0u  ) | ((Bitu)total[(i >> 2u) & 1u] << 8u  ) |
+                    ((Bitu)total[(i >> 1u) & 1u] << 16u ) | ((Bitu)total[(i >> 0u) & 1u] << 24u );
+#endif
+            }
+        }
     }
 
     Section_prop * section2=static_cast<Section_prop *>(control->GetSection("vsync"));
@@ -1438,6 +1493,16 @@ void VGA_OnEnterPC98_phase2(Section *sec) {
     /* GDC 2.5/5.0MHz setting is also reflected in BIOS data area and DIP switch registers */
     gdc_5mhz_mode_update_vars();
 
+    /* ARTIC (A Relative Time Indication Counter) at 0x5C and 0x5E */
+    if (pc98_timestamp5c) {
+        IO_RegisterReadHandler(0x5C,pc98_read_artic,IO_MB);
+        IO_RegisterReadHandler(0x5D,pc98_read_artic,IO_MB);
+        IO_RegisterReadHandler(0x5E,pc98_read_artic,IO_MB);
+        IO_RegisterReadHandler(0x5F,pc98_read_artic,IO_MB);
+        IO_RegisterReadHandler(0x5C,pc98_read_artic,IO_MW);
+        IO_RegisterReadHandler(0x5E,pc98_read_artic,IO_MW);
+    }
+
     /* delay I/O port at 0x5F (0.6us) */
     IO_RegisterWriteHandler(0x5F,pc98_wait_write,IO_MB);
 
@@ -2070,7 +2135,7 @@ public:
 	{}
 
 private:
-	virtual void getBytes(std::ostream& stream)
+	void getBytes(std::ostream& stream) override
 	{
 		uint32_t tandy_drawbase_idx, tandy_membase_idx;
 
@@ -2188,7 +2253,7 @@ private:
 		POD_Save_VGA_XGA(stream);
 	}
 
-	virtual void setBytes(std::istream& stream)
+	void setBytes(std::istream& stream) override
 	{
 		uint32_t tandy_drawbase_idx, tandy_membase_idx;
 
